zn.define(function () {
    var node_officegen = require('officegen'),
        node_xlsx = require('xlsx');

    return zn.Controller('wfindextable', {
        methods: {
            orderField: {
                method: 'GET/POST',
                argv: {
                    field_id: null,
                    order: 'up'
                },
                value: function (request, response, chain){
                    var _field_id = request.getValue('field_id'),
                        _order = request.getValue('order'),
                        _field = null;
                    this.beginTransaction()
                        .query(zn.sql.select({
                            table: 'zn_plugin_workflow_index_table_field',
                            where: { id: _field_id }
                        }))
                        .query('validate', function (sql, data){
                            _field = data[0];
                            if(!_field){
                                return response.error('未查到该字段'), false;
                            }

                            if(_order=='up'){
                                if(_field.field_order==1){
                                    return response.error('已经是最高'), false;
                                }else {
                                    return zn.sql.update({
                                        table: 'zn_plugin_workflow_index_table_field',
                                        updates: { field_order: _field.field_order },
                                        where: { field_order: _field.field_order - 1, event_id: _field.event_id }
                                    }) + zn.sql.update({
                                        table: 'zn_plugin_workflow_index_table_field',
                                        updates: {field_order: (_field.field_order - 1)},
                                        where: { id: _field_id }
                                    });
                                }
                            }else {
                                return zn.sql.update({
                                    table: 'zn_plugin_workflow_index_table_field',
                                    updates: { field_order: _field.field_order },
                                    where: { field_order: _field.field_order + 1, event_id: _field.event_id }
                                }) + zn.sql.update({
                                    table: 'zn_plugin_workflow_index_table_field',
                                    updates: {field_order: (_field.field_order + 1)},
                                    where: { id: _field_id }
                                });
                            }
                        }, function (err, data){
                            if(err){
                                return response.error(err);
                            }else {
                                return response.success("排序成功");
                            }
                        }).commit();
                }
            },
            exportTableExcelTemplate: {
                method: 'GET/POST',
                argv: {
                    tableId: null
                },
                value: function (request, response, chain){
                    var _tableId = request.getValue('tableId'),
                        _isTitle = request.getInt('isTitle');
                    this.beginTransaction()
                        .query(zn.sql.select({
                            table: 'zn_plugin_workflow_index_table',
                            where: {
                                id: _tableId
                            }
                        }) + zn.sql.select({
                            table: 'zn_plugin_workflow_index_table_field',
                            where: "table_id=" + _tableId + " and type<>'Hidden'",
                            order: {
                                field_order: 'asc'
                            }
                        }), null, function (err, rows){
                            if(err){
                                response.error(err);
                            }else {
                                var _table = rows[0][0],
                                    _fields = rows[1],
                                    _title = _isTitle ? _table.zn_title : _table.table_name;
                                    _xlsx = node_officegen('xlsx'),
                                    _sheet0 = _xlsx.makeNewSheet();
                                _sheet0.name = _table.zn_title;
                                _sheet0.data[0] = [];
                                _fields.forEach(function (field){
                                    _sheet0.data[0].push(field.title);
                                });
                                response.writeHead(200, {
                				    "Content-Type": "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
                				    'Content-disposition': 'attachment; filename=' + encodeURI(_title) + '_import_template.xlsx'
                				});
                                _xlsx.generate(response._serverResponse);
                            }
                        })
                        .commit();
                }
            },
            importTableExcelData: {
                method: 'GET/POST',
                argv: {
                    tableId: null,
                    start: 1,
                    vars: '{}',
                    distincts: '{}'
                },
                value: function (request, response, chain){
                    var _files = request.$files,
                        _result = [],
                        _tableId = request.getValue('tableId'),
                        _start = request.getInt('start'),
                        _vars = JSON.parse(request.getValue('vars')||'{}'),
                        _distincts = JSON.parse(request.getValue('vars')||'{}');

                    this.beginTransaction()
                        .query(zn.sql.select({
                            table: 'zn_plugin_workflow_index_table_field',
                            where: "table_id=" + _tableId + " and type<>'Hidden'",
                            order: {
                                field_order: 'asc'
                            }
                        }), null, function (err, rows){
                            if(err){
                                response.error(err);
                            }else {
                                var _titleNames = {},
                                    _indexNames = {};
                                for(var i = 0, _len = rows.length; i<_len; i++){
                                    _titleNames[rows[i].title] = rows[i].name;
                                }
                                zn.each(_files, function (file, key){
                                    var _file = request.uploadFile(file),
                                        _worksheet = node_xlsx.readFile(_file.path),
                                        _data = [],
                                        _rows = [];
                                    Object.keys(_worksheet.Sheets).map((name)=>{
                                        _data = node_xlsx.utils.sheet_to_json(_worksheet.Sheets[name], {
                                            header: 1,
                                            raw: true
                                        });

                                        if(_data.length){
                                            _data[_start-1].forEach(function (title, index){
                                                _indexNames[index] = _titleNames[title];
                                            });

                                            zn.each(_data, function (item, index){
                                                if(index > (_start-1)){
                                                    if(item.length){
                                                        var _values = {},
                                                            _value = null,
                                                            _name = null;
                                                        for(var i = 0; i < item.length; i++){
                                                            _value = item[i]||'';
                                                            _name = _indexNames[i];
                                                            if(!_name){
                                                                continue;
                                                            }
                                                            if(_vars[i]){
                                                                _value = _vars[i][_value];
                                                            }

                                                            _values[_name] = _value;
                                                        }
                                                        _values._editable = true;
                                                        _rows.push(_values);
                                                    }
                                                }
                                            });
                                        }

                                        _result.push({
                                            title: name,
                                            data: _data,
                                            values: _rows
                                        });
                                    });
                                });

                                response.success(_result);
                            }
                        })
                        .commit();
                }
            },
            generateTable: {
                method: 'GET/POST',
                argv: {
                    tableId: null
                },
                value: function (request, response, chain){
                    var _tableId = request.getValue('tableId');
                    this.beginTransaction()
                        .query(zn.sql.select({
                            table: 'zn_plugin_workflow_index_table',
                            where: {
                                id: _tableId
                            }
                        }) + zn.sql.select({
                            table: 'zn_plugin_workflow_index_table_field',
                            where: "table_id=" + _tableId,
                            order: {
                                field_order: 'asc'
                            }
                        }))
                        .query('create table', function (sql, rows){
                            var _table = rows[0][0];
                            if(_table){
                                var _fields = rows[1], __fields = {};
                                var _mixins = _table.mixins.split(','), __mixins = [];
                                if(_table.parent_id){
                                    __fields['zn_plugin_workflow_primary_id'] = {
                                        type: ["int", 11],
                                        default: 0
                                    };
                                }
                                __fields['zn_plugin_workflow_index_id'] = __fields['zn_plugin_workflow_instance_id'] = __fields['zn_plugin_workflow_ticket_status'] = {
                                    type: ["int", 11],
                                    default: 0
                                };
                                _mixins.forEach(function (value){
                                    if(!!value){
                                        __mixins.push(zn.path(global, value));
                                    }
                                });
                                _fields.map(function(field){
                                    __fields[field.name] = {
                                        type: JSON.parse(field.field_type),
                                        default: field.field_default
                                    };
                                });
                                return zn.Model(_table.table_name, {
                                    mixins: __mixins,
                                    properties: __fields
                                }).getCreateSql() + zn.sql.update({
                                    table: 'zn_plugin_workflow_index_table',
                                    where: { id: _tableId },
                                    updates: { has_generated: 1 }
                                });
                            }else {
                                return response.error('表存在'), false;
                            }
                        }, function (err, rows){
                            if(err){
                                response.error(err);
                            }else {
                                response.success(rows);
                            }
                        }).commit();
                }
            }
        }
    });

});
