zn.define(function () {

    var CODE_FORMAT = {
        NOW_YMD: function (sep){
            var _now = new Date(),
                _m = _now.getMonth() + 1,
                _d = _now.getDate();
            _m = _m<10?('0'+_m):_m;
            _d = _d<10?('0'+_d):_d;
            return [_now.getFullYear(), _m, _d].join(sep||'');
        },
        NOW_HMS: function (){
            var _now = new Date();
            return [_now.getFullYear(), _now.getMonth() + 1, _now.getDate()].join(sep||'');
        }
    };

    return zn.Class({
        methods: {
            init: function (store){
                this._store = store;
            },
            formatCode: function (format){

            },
            getSessionId: function () {
                if(zn._request){
                    return zn._request.getSessionValueByKey('id');
                }else if(zn._oldRequest) {
                    return zn._oldRequest.getSessionValueByKey('id');
                }else {
                    return 0;
                }
            },
            createInstance: function (indexId, options, data){
                var _defer = zn.async.defer(),
                    _value = {},
                    _options = zn.extend({
                        parent_id: 0,
                    }, options),
                    _sessionId = this.getSessionId(),
                    _index = null
                    _instanceId = 0,
                    _begin = null,
                    _rights = {},
                    _formData = data || {},
                    _formSql = [];
                if(!_sessionId){
                    return _defer.reject(new Error("登录Session已失效, 请重新登录!")), _defer.promise;
                }
                this._store.beginTransaction()
                    .query('select wfindex json_schema', function (){
                        return "select * from zn_plugin_workflow_schema where type='begin' and index_id={0};select zn_title, code_format from zn_plugin_workflow_index where id={0};".format(indexId);
                    })
                    .query('create instance parent', function (sql, rows){
                        _begin = rows[0][0];
                        _index = rows[1][0];
                        _options.zn_title = _options.zn_title || '【 ' + _index.zn_title + ' 】';
                        if(!_begin){
                            return _defer.reject(new Error('流程中未定义开始节点, 请在流程定义处确认!')), false;
                        }

                        _rights = {
                            zn_rights_owner_id: _begin.zn_rights_owner_id,
                            zn_rights_users: _begin.zn_rights_observe_users,
                            zn_rights_roles: _begin.zn_rights_observe_roles,
                            zn_rights_observe_users: _begin.zn_rights_observe_users,
                            zn_rights_observe_roles: _begin.zn_rights_observe_roles
                        };
                        switch (_begin.rights_logic) {
                            case 'rights.logic.create.person':
                                _rights.zn_rights_users = ',' + _sessionId + ',';
                                _rights.zn_rights_roles = ',';
                                break;
                            case 'rights.logic.user.role.owners':
                                _rights.zn_rights_users = '{zn_plugin_admin_user_role_owners('+_sessionId+')}';
                                _rights.zn_rights_roles = ',';
                                break;
                            case 'rights.logic.default.role.owners':
                                _rights.zn_rights_users = '{zn_plugin_admin_role_owners(' + _begin.zn_rights_observe_roles + ')}';
                                _rights.zn_rights_roles = ',';
                                break;
                        }
                        return zn.sql.insert({
                            table: 'zn_plugin_workflow_instance',
                            values: zn.extend({
                                code: "CN-" + CODE_FORMAT.NOW_YMD() + '-' + Math.random().toString().slice(-6),
                                index_id: indexId,
                                zn_rights_enabled: 1,
                                state: 1
                            }, _rights, _options)
                        });
                    })
                    .query('create instance', function (sql, data){
                        _instanceId = data.insertId;
                        if(_instanceId){
                            _value.instance_id = _instanceId;
                            var _item = null,
                                _sql = '';
                            for(var table in _formData){
                                _item = _formData[table];
                                if(zn.is(_item, 'array')){
                                    zn.each(_item, function (values){
                                        values.zn_plugin_workflow_primary_id = "{0}";
                                        values.zn_plugin_workflow_index_id = indexId;
                                        values.zn_plugin_workflow_instance_id = _instanceId;
                                        values._editable = null;
                        				delete values._editable;
                                        _formSql.push(zn.sql.insert({
                                            table: table,
                                            values: values
                                        }));
                                    });
                                }else {
                                    _item.zn_plugin_workflow_index_id = indexId;
                                    _item.zn_plugin_workflow_instance_id = _instanceId;
                                    _sql = zn.sql.insert({
                                        table: table,
                                        values: _item
                                    });
                                }
                            }

                            return zn.sql.insert({
                                table: 'zn_plugin_workflow_instance',
                                values: zn.extend({
                                    index_id: indexId,
                                    zn_rights_enabled: 1,
                                    state: 1,
                                    parent_id: _instanceId,
                                    rights_logic: _begin.rights_logic,
                                    next_schemas: _begin.next_nodes,
                                    schema_id: _begin.id,
                                    schema_uuid: _begin.uuid,
                                    type: _begin.type,
                                    zn_title: _begin.zn_title,
                                    icon: _begin.icon,
                                    attrs: _begin.attrs
                                }, _rights)
                            }) + _sql;
                        }else {
                            return _defer.reject(new Error('流程创建失败!')), false;
                        }
                    })
                    .query('update instance', function (sql, data){
                        if(zn.is(data, 'array')){
                            var _primaryId = data[1].insertId;
                            _value.primary_id = _primaryId;
                            return zn.sql.update({
                                table: 'zn_plugin_workflow_instance',
                                updates: {
                                    form_id:  _primaryId
                                },
                                where: "id=" + _instanceId
                            }) + _formSql.join('').format(_primaryId);
                        }else {
                            return zn.sql.select({
                                table: 'zn_plugin_workflow_instance',
                                where: "id=" + data[0].insertId
                            });
                        }
                    }, function (err, rows){
                        if(err){
                            _defer.reject(err);
                        }else {
                            _defer.resolve(_value);
                        }
                    }).commit();

                return _defer.promise;
            },
            toDefinition: function (indexId){
                var _defer = zn.async.defer();
                var _insert_schema_sql = "insert into zn_plugin_workflow_schema (index_id, uuid, zn_title, type, previous_nodes, next_nodes, previous_ids, next_ids) values ("+indexId+", '{0}', '{1}', '{2}', '{3}', '{4}', '{5}', '{6}');";
                var _schema = null;
                this._store.beginTransaction()
                    .query('select wfindex json_schema', function (){
                        return "select json_schema from zn_plugin_workflow_index where id={0};delete from zn_plugin_workflow_schema where index_id={0};".format(indexId);
                    })
                    .query('insert schema', function (sql, rows){
                        var _index = rows[0][0];
                        if(!_index||(_index&&!_index.json_schema)){
                            return _defer.reject(new Error('流程还未定义')), false;
                        }else {
                            _schema = JSON.parse(_index.json_schema) || { nodes: [], links: [] };
                        }
                        var _schema_sqls = '',
                            _ids = {},
                            _source = null,
                            _target = null;

                        _schema.links.forEach(function (link){
                            _source = link.source;
                            _target = link.target;
                            _ids[_source] = _ids[_source] || { previous_nodes: ',', next_nodes: ',' };
                            _ids[_target] = _ids[_target] || { previous_nodes: ',', next_nodes: ',' };
                            _ids[_source].next_nodes = _ids[_source].next_nodes + _target + ',';
                            _ids[_target].previous_nodes = _ids[_target].previous_nodes + _source + ',';
                        });

                        _schema.nodes.forEach(function (node){
                            _schema_sqls += _insert_schema_sql.format(node.id, node.data.title, node.data.type, _ids[node.id].previous_nodes, _ids[node.id].next_nodes, _ids[node.id].previous_nodes, _ids[node.id].next_nodes);
                        });

                        return _schema_sqls;
                    })
                    .query("select * from zn_plugin_workflow_schema where index_id={0};".format(indexId))
                    .query('update ref', function (sql, rows){
                        var _obj = {},
                            _ids = [],
                            _next_ids = 'next_ids',
                            _pre_ids = 'previous_ids';
                        rows.forEach(function (row, index){
                            _obj[row.uuid] = row.id;
                            _ids.push(row.id);
                        });

                        for(var key in _obj){
                            _next_ids = "replace(" + _next_ids + ",'" + key + "','" + _obj[key] + "')";
                            _pre_ids = "replace(" + _pre_ids + ",'" + key + "','" + _obj[key] + "')";
                        }

                        return "update zn_plugin_workflow_schema set next_ids={0}, previous_ids={1} where id in ({2});".format(_next_ids, _pre_ids, _ids.join(','));
                    }, function (err, rows){
                        if(err){
                            _defer.reject(err);
                        }else {
                            _defer.resolve(rows);
                        }
                    }).commit();

                return _defer.promise;
            }
        }
    });

});
