zn.define([
    './WFIndexService',
    './WFSchemaService'
], function (WFIndexService, WFSchemaService) {

    var STATE = {
        GOING: 1,
        PASSED: 2,
        ENDED: -1,
        CANCELED: -2,
        P_GOING: 100,
        P_PASS: 101,
        P_DENY: -101
    }

    var TYPE = {
        BEGIN: 'begin',
        NORMAL: 'normal',
        PARALLEL: 'parallel',
        CANCEL: 'cancel',
        AUTO: 'auto',
        SUBFLOW: 'subflow',
        END: 'end'
    }

    return zn.Class({
        methods: {
            init: function (store){
                this._store = store;
                this._indexService = new WFIndexService(store);
                this._schemaService = new WFSchemaService(store);
            },
            getSessionId: function (){
                if(zn._request){
                    return zn._request.getSessionValueByKey('id');
                }else if(zn._oldRequest) {
                    return zn._oldRequest.getSessionValueByKey('id');
                }else {
                    return 0;
                }
            },
            start: function (instanceId){
                var _defer = zn.async.defer();
                if(!this.getSessionId()){
                    return _defer.reject(new Error("登录Session已失效, 请重新登录!")), _defer.promise;
                }
                this._store.beginTransaction()
                    .query('select workflow instance', function (){
                        return "select * from zn_plugin_workflow_instance where id={0} and closed=1;".format(instanceId);
                    })
                    .query('cancel instance', function (sql, rows){
                        if(!rows[0]){
                            return _defer.reject('流程已关闭或不存在!'), false;
                        }
                        return zn.sql.update({
                            table: 'zn_plugin_workflow_instance',
                            updates: {
                                closed: 0
                            },
                            where: {
                                id: instanceId
                            }
                        });
                    }, function (err, rows){
                        if(err){
                            _defer.reject(err);
                        }else {
                            _defer.resolve(rows);
                        }
                    }).commit();

                return _defer.promise;
            },
            remove: function (instanceId){
                var _defer = zn.async.defer();
                if(!this.getSessionId()){
                    return _defer.reject(new Error("登录Session已失效, 请重新登录!")), _defer.promise;
                }
                this._store.beginTransaction()
                    .query('select workflow instance', function (){
                        return "select * from zn_plugin_workflow_instance where id={0};".format(instanceId);
                    })
                    .query('select index', function (sql, rows){
                        if(!rows[0]){
                            return _defer.reject('流程不存在!'), false;
                        }
                        return zn.sql.select({
                            table: 'zn_plugin_workflow_index_table',
                            where: {
                                index_id: rows[0].index_id
                            }
                        }) + zn.sql.delete({
                            table: 'zn_plugin_workflow_instance',
                            where: 'id=' + instanceId + " or parent_id=" + instanceId
                        }) + zn.sql.delete({
                            table: 'zn_plugin_workflow_instance',
                            where: {
                                id: instanceId
                            }
                        });
                    })
                    .query('delete instance data', function (sql, rows){
                        var _tables = rows[0],
                            _sqls = [];
                        _tables.forEach(function (table){
                            _sqls.push(zn.sql.delete({
                                table: table.table_name,
                                where: { zn_plugin_workflow_instance_id: instanceId }
                            }));
                        });
                        if(!_sqls.length){
                            return response.success('删除成功'), false;
                        }
                        return _sqls.join('');
                    }, function (err, rows){
                        if(err){
                            _defer.reject(err);
                        }else {
                            _defer.resolve(rows);
                        }
                    }).commit();

                return _defer.promise;
            },
            cancel: function (instanceId){
                var _defer = zn.async.defer();
                if(!this.getSessionId()){
                    return _defer.reject(new Error("登录Session已失效, 请重新登录!")), _defer.promise;
                }
                this._store.beginTransaction()
                    .query('select workflow instance', function (){
                        return "select * from zn_plugin_workflow_instance where id={0} and closed=0;".format(instanceId, STATE.CANCELED);
                    })
                    .query('cancel instance', function (sql, rows){
                        if(!rows[0]){
                            return _defer.reject(new Error('流程已关闭或不存在!')), false;
                        }
                        return zn.sql.update({
                            table: 'zn_plugin_workflow_instance',
                            updates: {
                                closed: 1
                            },
                            where: {
                                id: instanceId
                            }
                        });
                    }, function (err, rows){
                        if(err){
                            _defer.reject(err);
                        }else {
                            _defer.resolve(rows);
                        }
                    }).commit();

                return _defer.promise;
            },
            deleteInstance: function (instanceId){
                var _defer = zn.async.defer();
                if(!this.getSessionId()){
                    return _defer.reject(new Error("登录Session已失效, 请重新登录!")), _defer.promise;
                }
                this._store.beginTransaction()
                    .query('delete workflow instance', function (sql, data){
                        return "delete from zn_plugin_workflow_instance where id={0};delete from zn_plugin_workflow_instance where parent_id={0}".format(instanceId);
                    }, function (err, rows){
                        if(err){
                            _defer.reject(err);
                        }else {
                            _defer.resolve(rows);
                        }
                    }).commit();

                return _defer.promise;
            },
            next: function (instanceId, nextSchemaUUID, options){
                var _defer = zn.async.defer();
                var _current = null,
                    _instance = null;
                if(!this.getSessionId()){
                    return _defer.reject(new Error("登录Session已失效, 请重新登录!")), _defer.promise;
                }
                this._store
                    .beginTransaction()
                    .query("select instance and current schema", function (sql){
                        return 'select state, index_id from zn_plugin_workflow_instance where closed=0 and id={0};select type, schema_uuid from zn_plugin_workflow_instance where state=1 and parent_id={0};'.format(instanceId);
                    })
                    .query("select instance table", function (sql, data){
                        _instance = data[0][0];
                        _current = data[1][0];
                        if(!_instance){
                            return _defer.reject(new Error('流程实例不存在或者已被关闭')), false;
                        }

                        if(_current.schema_uuid == nextSchemaUUID){
                            return _defer.reject(new Error('请勿重复调用')), false;
                        }

                        if(_instance.state == STATE.ENDED || _current.type == TYPE.END){
                            return _defer.reject(new Error('流程实例已经结束')), false;
                        }

                        if(_instance.state == STATE.CANCELED || _current.type == TYPE.CANCEL){
                            return _defer.reject(new Error('流程实例已经被取消')), false;
                        }

                        return "select * from zn_plugin_workflow_index_table where index_id={0} and has_generated=1 and parent_id=0;".format(_instance.index_id);
                    })
                    .query("select data", function (sql, rows){
                        var _table = rows[0];
                        var _sql = "select * from zn_plugin_workflow_schema_action where schema_uuid='{0}' and _order='__after';".format(_current.schema_uuid);
                        _sql += "select * from zn_plugin_workflow_schema_action where schema_uuid='{0}' and _order='__before';".format(nextSchemaUUID);
                        if(_table){
                            _sql += "select * from {0} where zn_plugin_workflow_instance_id={1};".format(_table.table_name, instanceId);
                        }

                        return _sql;
                    }, function (err, rows){
                        var _afterActions = rows[0],
                            _beforeActions = rows[1],
                            _form = null,
                            _obj = {
                                current: _current,
                                instance: _instance
                            };
                        if(rows[2]){
                            _form = rows[2][0];
                        }

                        if(err){
                            _defer.reject(err);
                        }else {
                            switch (_current.type) {
                                case TYPE.BEGIN:
                                case TYPE.NORMAL:
                                    this.__nextNormal(instanceId, nextSchemaUUID, options, _obj)
                                        .then(function(data){
                                            _defer.resolve(data);
                                            this._schemaService.doActions([].concat(_afterActions, _beforeActions), _form, instanceId);
                                        }.bind(this), function (err){
                                            _defer.reject(err);
                                        });
                                    break;
                                case TYPE.PARALLEL:
                                    this.__nextParallel(instanceId, nextSchemaUUID, options, _obj)
                                        .then(function(data){
                                            _defer.resolve(data);
                                            this._schemaService.doActions([].concat(_afterActions, _beforeActions), _form, instanceId);
                                        }.bind(this), function (err){
                                            _defer.reject(err);
                                        });
                                    break;
                                case TYPE.AUTO:
                                    this.__nextAuto(instanceId, nextSchemaUUID, options, _obj)
                                        .then(function(data){
                                            _defer.resolve(data);
                                            this._schemaService.doActions([].concat(_afterActions, _beforeActions), _form, instanceId);
                                        }.bind(this), function (err){
                                            _defer.reject(err);
                                        });
                                    break;
                                case TYPE.SUBFLOW:
                                    this.__nextSubflow(instanceId, nextSchemaUUID, options, _obj)
                                        .then(function(data){
                                            _defer.resolve(data);
                                            this._schemaService.doActions([].concat(_afterActions, _beforeActions), _form, instanceId);
                                        }.bind(this), function (err){
                                            _defer.reject(err);
                                        });
                                    break;
                                default:
                                    _defer.reject(new Error('node type is ' + _curr.type));
                                    break;
                            }
                        }
                    }.bind(this))
                    .commit();

                return _defer.promise;
            },
            __nextNormal: function (instanceId, nextSchemaUUID, options){
                var _defer = zn.async.defer();
                var _sql = '',
                    _rights = null,
                    _instance = null,
                    _currNode = null,
                    _nextSchema = null,
                    _nextId = 0,
                    _sessionId = this.getSessionId(),
                    _parallel_users = ',';
                this._store
                    .beginTransaction()
                    .query('select instance, currNode, nextSchema', function (sql){
                        _sql = "select * from zn_plugin_workflow_instance where id={0};".format(instanceId);
                        _sql += "select * from zn_plugin_workflow_instance where state=1 and parent_id={0};".format(instanceId);
                        _sql += "select * from zn_plugin_workflow_schema where uuid='{0}';".format(nextSchemaUUID);
                        return _sql;
                    })
                    .query('insert next and update instance', function (sql, data){
                        _instance = data[0][0];
                        _currNode = data[1][0];
                        _nextSchema = data[2][0];
                        if(!_instance || !_currNode || !_nextSchema){
                            return _defer.reject(new Error('instanceId or nextSchemaUUID is invalid.')), false;
                        }
                        _rights = this.__getRights(_instance, _currNode, _nextSchema);
                        if(_nextSchema.type != 'end' && _rights.zn_rights_users.length < 2 && _rights.zn_rights_roles.length < 2){
                            return _defer.reject(new Error('The next step has no invalid rights, you have to set it on workflow designer.')), false;
                        }
                        _sql = zn.sql.insert({
                            table: 'zn_plugin_workflow_instance',
                            values: {
                                state: STATE.GOING,
                                previous: _currNode.id,
                                index_id: _instance.index_id,
                                parent_id: _instance.id,
                                rights_logic: _nextSchema.rights_logic,
                                next_schemas: _nextSchema.next_nodes,
                                schema_id: _nextSchema.id,
                                schema_uuid: _nextSchema.uuid,
                                type: _nextSchema.type,
                                zn_title: _nextSchema.zn_title,
                                icon: _nextSchema.icon,
                                attrs: _nextSchema.attrs,
                                zn_rights_users: _rights.zn_rights_users,
                                zn_rights_roles: _rights.zn_rights_roles,
                                zn_rights_owner_id: _nextSchema.zn_rights_owner_id,
                                zn_rights_observe_users: _nextSchema.zn_rights_observe_users,
                                zn_rights_observe_roles: _nextSchema.zn_rights_observe_roles
                            }
                        });

                        if(_nextSchema.type == TYPE.PARALLEL){
                            _sql += "select concat('"+_rights.zn_rights_users+"', zn_plugin_admin_role_owners('"+_rights.zn_rights_roles+"')) as users;";
                        }else {
                            _sql += "select ',' as users;";
                        }

                        return _sql + "select table_name from zn_plugin_workflow_index_table where parent_id=0 and index_id=" + _instance.index_id;
                    }.bind(this))
                    .query('update currNode', function (sql, data){
                        _nextId = data[0].insertId;
                        _parallel_users = data[1][0].users;
                        _sql = zn.sql.update({
                            table: 'zn_plugin_workflow_instance',
                            updates: zn.extend({
                                state: STATE.PASSED,
                                next: _nextId
                            }, options),
                            where: {
                                id: _currNode.id
                            }
                        });

                        _sql += zn.sql.update({
                            table: 'zn_plugin_workflow_instance',
                            updates: {
                                state: _nextSchema.type == TYPE.END ? STATE.ENDED: STATE.GOING,
                                do_users: "{{replace(concat(do_users, '" + _sessionId + ",'), ',,', ',')}}",
                                zn_modify_time: "{{now()}}",
                                zn_rights_users: _rights.zn_rights_users,
                                zn_rights_roles: _rights.zn_rights_roles,
                                zn_rights_observe_users: "{{replace(concat(zn_rights_observe_users, '" + _nextSchema.zn_rights_observe_users + "'), ',,', ',')}}",
                                zn_rights_observe_roles: "{{replace(concat(zn_rights_observe_roles, '" + _nextSchema.zn_rights_observe_roles + "'), ',,', ',')}}"
                            },
                            where: {
                                id: _instance.id
                            }
                        });

                        if(data[2][0]){
                            _sql += zn.sql.update({
                                table: data[2][0].table_name,
                                updates: { zn_plugin_workflow_ticket_status: -1 },
                                where: {
                                    zn_plugin_workflow_instance_id: _instance.id
                                }
                            });
                        }

                        return _sql;
                    }.bind(this))
                    .query('do type', function (sql, data){
                        switch (_nextSchema.type) {
                            case TYPE.PARALLEL:
                                var _ids = {},
                                    _sqls = [];
                                _parallel_users.split(',').map(function (value){
                                    if(!_ids[value] && value){
                                        _ids[value] = value;
                                        _sqls.push(zn.sql.insert({
                                            table: 'zn_plugin_workflow_instance',
                                            values: {
                                                state: STATE.P_GOING,
                                                parent_id: _nextId,
                                                zn_title: value,
                                                zn_rights_users: "," + value + ","
                                            }
                                        }));
                                    }
                                });
                                return _sqls.join('');
                            case TYPE.SUBFLOW:
                                return this._indexService.createInstance(_nextSchema.sub_index_id, {
                                    parent_id: _nextId,
                                    zn_title: _instance.zn_title + ' / ' + _nextSchema.zn_title + ' /【子流程】'
                                }).then(function (data){
                                    _defer.resolve(data);
                                }, function (err){
                                    _defer.reject(err);
                                }), -1;
                                break;
                            default:
                                return _defer.resolve('流程流转成功'), -1;
                        }
                    }, function (err, data){
                        if(err){
                            _defer.reject(err);
                        }else {
                            _defer.resolve(data);
                        }
                    })
                    .commit();

                return _defer.promise;
            },
            __nextParallel: function (instanceId, nextSchemaUUID, options, data){
                var _defer = zn.async.defer();
                var _sql = '',
                    _sessionId = this.getSessionId();
                this._store
                    .beginTransaction()
                    .query('update instance, all child', function (sql){
                        _sql = zn.sql.update({
                            table: 'zn_plugin_workflow_instance',
                            updates: {
                                zn_rights_users: "{REPLACE(zn_rights_users, '" + _sessionId + ",', '')}"
                            },
                            where: {
                                id: instanceId
                            }
                        });
                        _sql = zn.sql.update({
                            table: 'zn_plugin_workflow_instance',
                            updates: zn.extend({
                                state: STATE.P_PASS
                            }, options),
                            where: "parent_id=" + data.current.id + " and zn_plugin_admin_user_confirm(zn_title,'" + _sessionId + "')<>0"
                        });

                        return _sql;
                    })
                    .query('select count', function (){
                        return "select count(*) as count from zn_plugin_workflow_instance where parent_id={0} and (state={1} or state={2});".format(data.current.id, STATE.P_GOING, STATE.P_DENY);
                    }, function (err, data){
                        if(err){
                            _defer.reject(err);
                        }else {
                            if(data[0].count){
                                _defer.resolve('流程流转成功');
                            }else {
                                this.__nextNormal(instanceId, nextSchemaUUID, {})
                                    .then(function (data){
                                        _defer.resolve(data);
                                    }, function (err){
                                        _defer.reject(err);
                                    });
                            }
                        }
                    }.bind(this))
                    .commit();

                return _defer.promise;
            },
            __denyParallel: function (instanceId, nextSchemaUUID, options, data){
                var _defer = zn.async.defer();
                var _sessionId = this.getSessionId();
                this._store
                    .beginTransaction()
                    .query('update instance, all child', function (sql){
                        return zn.sql.update({
                            table: 'zn_plugin_workflow_instance',
                            updates: zn.extend({
                                state: STATE.P_PASS
                            }, options),
                            where: "parent_id=" + data.current.id + " and zn_plugin_admin_user_confirm(zn_title,'" + _sessionId + "')<>0"
                        });
                    })
                    .query('select count', function (){
                        return "select count(*) as count from zn_plugin_workflow_instance where parent_id={0} and (state={1} or state={2});".format(data.current.id, STATE.P_GOING, STATE.P_DENY);
                    }, function (err, data){
                        if(err){
                            _defer.reject(err);
                        }else {
                            if(data[0].count){
                                _defer.resolve('流程流转成功');
                            }else {
                                this.__nextNormal(instanceId, nextSchemaUUID, {})
                                    .then(function (data){
                                        _defer.resolve(data);
                                    }, function (err){
                                        _defer.reject(err);
                                    });
                            }
                        }
                    }.bind(this))
                    .commit();

                return _defer.promise;
            },
            __nextAuto: function (instanceId, nextSchemaUUID, options, data){
                var _defer = zn.async.defer();
                var _sql = '',
                    _sessionId = this.getSessionId();
                this._store
                    .beginTransaction()
                    .query('update instance, all child', function (sql){
                        _sql = zn.sql.update({
                            table: 'zn_plugin_workflow_instance',
                            updates: zn.extend({
                                state: STATE.P_PASS
                            }, options),
                            where: "parent_id=" + data.current.id + " and zn_plugin_admin_user_confirm(zn_title,'" + _sessionId + "')<>0"
                        });

                        return _sql;
                    })
                    .query('select count', function (){
                        return "select count(*) as count from zn_plugin_workflow_instance where parent_id={0} and (state={1} or state={2});".format(data.current.id, STATE.P_GOING, STATE.P_DENY);
                    }, function (err, data){
                        if(err){
                            _defer.reject(err);
                        }else {
                            if(data[0].count){
                                _defer.resolve('流程流转成功');
                            }else {
                                this.__nextNormal(instanceId, nextSchemaUUID, {})
                                    .then(function (data){
                                        _defer.resolve(data);
                                    }, function (err){
                                        _defer.reject(err);
                                    });
                            }
                        }
                    }.bind(this))
                    .commit();

                return _defer.promise;
            },
            __nextSubflow: function (instanceId, nextSchemaUUID, options, data){
                var _defer = zn.async.defer();
                var _sql = '',
                    _sessionId = this.getSessionId();
                this._store
                    .beginTransaction()
                    .query('select count', function (){
                        return "select count(*) as count from zn_plugin_workflow_instance where parent_id={0} and (state={1} or state={2});".format(data.current.id, STATE.P_GOING, STATE.P_DENY);
                    }, function (err, data){
                        if(err){
                            _defer.reject(err);
                        }else {
                            if(data[0].count){
                                _defer.resolve('流程流转成功');
                            }else {
                                this.__nextNormal(instanceId, nextSchemaUUID, {})
                                    .then(function (data){
                                        _defer.resolve(data);
                                    }, function (err){
                                        _defer.reject(err);
                                    });
                            }
                        }
                    }.bind(this))
                    .commit();

                return _defer.promise;
            },
            __getRights: function (instance, currNode, nextSchema) {
                var _rights = {},
                    _sessionId = this.getSessionId();
                switch (nextSchema.rights_logic) {
                    case 'rights.logic.default':
                        _rights.zn_rights_users = nextSchema.zn_rights_observe_users;
                        _rights.zn_rights_roles = nextSchema.zn_rights_observe_roles;
                        break;
                    case 'rights.logic.default.role.owners':
                        _rights.zn_rights_users = '{zn_plugin_admin_role_owners(' + nextSchema.zn_rights_observe_roles + ')}';
                        _rights.zn_rights_roles = ',';
                        break;
                    case 'rights.logic.default.select':
                        _rights.zn_rights_users = ',';
                        _rights.zn_rights_roles = ',';
                        break;
                    case 'rights.logic.system.select':
                        _rights.zn_rights_users = ',';
                        _rights.zn_rights_roles = ',';
                        break;
                    case 'rights.logic.create.person':
                        _rights.zn_rights_users = ',' + _sessionId + ',';
                        _rights.zn_rights_roles = ',';
                        break;
                    case 'rights.logic.previous.rights':
                        _rights.zn_rights_users = currNode.zn_rights_users;
                        _rights.zn_rights_roles = currNode.zn_rights_roles;
                        break;
                    case 'rights.logic.all.rights':
                        _rights.zn_rights_users = instance.zn_rights_observe_users;
                        _rights.zn_rights_roles = instance.zn_rights_observe_roles;
                        break;
                    case 'rights.logic.user.role.owners':
                        _rights.zn_rights_users = '{zn_plugin_admin_user_role_owners('+_sessionId+')}';
                        _rights.zn_rights_roles = ',';
                        break;
                }

                return _rights;
            }
        }
    });

});
