"use strict";

require('./WFSchemaDesigner.less');

var React = require('react');

var NODE_TYPES = {
  'begin': '开始节点',
  'end': '结束节点',
  'normal': '普通节点',
  'parallel': '并行节点',
  'auto': '自动节点',
  'cancel': '取消节点',
  'subflow': '子流程'
},
    TYPE_ARRAY = ['begin', 'normal', 'parallel', 'auto', 'cancel', 'subflow', 'end'].map(function (index) {
  return {
    text: NODE_TYPES[index],
    value: index
  };
}),
    RIGHTS_LOGICS = [{
  text: '默认权限',
  value: 'rights.logic.default'
}, {
  text: '默认权限部门负责人',
  value: 'rights.logic.default.role.owners'
}, {
  text: '默认权限中选择',
  value: 'rights.logic.default.select'
}, {
  text: '系统权限中选择',
  value: 'rights.logic.system.select'
}, {
  text: '发起人',
  value: 'rights.logic.create.person'
}, {
  text: '上一步参与人',
  value: 'rights.logic.previous.rights'
}, {
  text: '所有参与人',
  value: 'rights.logic.all.rights'
}, {
  text: '发起人所在部门负责人',
  value: 'rights.logic.user.role.owners'
}];
var SchemaAction = React.createClass({
  displayName: "SchemaAction",
  getDefaultProps: function getDefaultProps() {
    return {
      model: 'ZNPluginWorkflowSchemaAction'
    };
  },
  getInitialState: function getInitialState() {
    return {
      data: zn.store.post('/zn.plugin.admin/model/paging', {
        model: this.props.model,
        where: this.props.schema_uuid ? {
          schema_uuid: this.props.schema_uuid
        } : null
      }),
      items: [{
        title: '名称',
        name: 'zn_title',
        width: 120
      }, {
        title: '可用',
        name: 'disabled',
        width: 40
      }, {
        title: '时序',
        name: '_order',
        width: 60
      }, {
        title: '类型',
        name: '_type',
        width: 60
      }, {
        title: '键(_key)',
        name: '_key',
        width: 80
      }, {
        title: '值(_value)',
        name: '_value',
        width: 280
      }, {
        title: '参数',
        name: '_parameters',
        width: 200
      }, {
        title: '说明',
        name: 'zn_note'
      }],
      formItems: [{
        title: '名称',
        name: 'zn_title',
        type: 'Textarea'
      }, {
        title: '是否可用',
        name: 'disabled',
        data: [{
          text: "是",
          value: 0
        }, {
          text: "否",
          value: 1
        }],
        type: 'Radio'
      }, {
        title: '时序',
        name: '_order',
        type: 'Select',
        data: [{
          text: '前置',
          value: '__before'
        }, {
          text: '初始化',
          value: '__init'
        }, {
          text: '后置',
          value: '__after'
        }]
      }, {
        title: '类型',
        name: '_type',
        type: 'Select',
        data: [{
          text: '推送内部消息',
          value: '__message'
        }, {
          text: '请求链接',
          value: '__url'
        }, {
          text: '发送邮件',
          value: '__email'
        }, {
          text: '发送短信',
          value: '__sms'
        }]
      }, {
        title: '键(_key)',
        name: '_key',
        type: 'Input'
      }, {
        title: '值(_value)',
        name: '_value',
        type: 'Textarea'
      }, {
        title: '参数',
        name: '_parameters',
        type: 'Textarea'
      }, {
        title: '说明',
        name: 'zn_note',
        type: 'Textarea'
      }],
      toolbarItems: [{
        text: '添加',
        method: '__addItem',
        icon: 'fa-plus',
        style: {
          marginRight: 5
        }
      }, {
        text: '删除',
        method: '__removeItems',
        status: 'danger',
        icon: 'fa-remove',
        style: {
          marginRight: 5
        }
      }]
    };
  },
  componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
    if (nextProps.schema_uuid) {
      this.state.data.extend({
        where: {
          schema_uuid: nextProps.schema_uuid
        }
      }).refresh();
    }
  },
  __doSuccess: function __doSuccess() {
    this.state.data.refresh();
    zn.notification.success('操作成功');
    zn.modal.close();
    return false;
  },
  __addItem: function __addItem(event) {
    /*event.stopPropagation();
    zn.popover.render(<zn.react.Form
    	action='/zn.plugin.admin/model/insert'
    	merge='values'
    	hiddens={{ schema_uuid: this.props.schema_uuid }}
    	exts={{ model: this.props.model }}
    	onSubmitSuccess={this.__doSuccess}
    	items={this.state.formItems} />, zn.extend({
    	event: event,
    	reset: true,
    	target: event.target,
    	style: {overflow: 'auto'}
    }, this.props));*/
    zn.dialog({
      title: '新增',
      content: React.createElement(zn.react.Form, {
        action: "/zn.plugin.admin/model/insert",
        merge: "values",
        hiddens: {
          schema_uuid: this.props.schema_uuid
        },
        exts: {
          model: this.props.model
        },
        onSubmitSuccess: this.__doSuccess,
        items: this.state.formItems
      })
    });
  },
  __updateItem: function __updateItem(data) {
    zn.dialog({
      title: '更新',
      content: React.createElement(zn.react.Form, {
        merge: "updates",
        action: "/zn.plugin.admin/model/update",
        exts: {
          model: this.props.model,
          where: {
            id: data.id
          }
        },
        value: data,
        onSubmitSuccess: this.__doSuccess,
        items: this.state.formItems
      })
    });
  },
  __removeItems: function __removeItems() {
    var _self = this,
        _values = this.refs.table.getValue();

    if (_values && _values.length) {
      zn.confirm('确认删除这' + _values.length + '个数据吗？', '提示', function () {
        zn.http.post('/zn.plugin.admin/model/delete', {
          model: this.props.model,
          where: "id in (" + _values.join(',') + ")"
        }).then(function (data) {
          if (data.status == 200) {
            zn.notification.success('删除成功');

            _self.state.data.refresh();
          } else {
            zn.notification.error('调用失败: ' + data.result);
          }
        }, function () {
          zn.notification.error('服务器错误: 删除失败');
        });
      }.bind(this));
    } else {
      zn.notification.warning('请先选择要删除的数据');
    }
  },
  __onToolbarClick: function __onToolbarClick(item, icon, icons, event) {
    this[item.method] && this[item.method](event);
  },
  __onTableColumnRender: function __onTableColumnRender(rowIndex, columnIndex, data, item, value) {
    var _this = this;

    switch (columnIndex) {
      case 1:
        return React.createElement("span", null, React.createElement("i", {
          className: "fa fa-edit zr-padding-3",
          onClick: function onClick() {
            return _this.__updateItem(data);
          }
        }), value);

      case 2:
        return value == 1 ? '否' : '是';
    }

    return value;
  },
  render: function render() {
    return React.createElement(zn.react.PagerView, {
      ref: "table",
      view: "Table",
      checkbox: 50,
      showHeader: true,
      toolbarItems: this.state.toolbarItems,
      onToolbarClick: this.__onToolbarClick,
      columnRender: this.__onTableColumnRender,
      data: this.state.data,
      dataFixed: true,
      items: this.state.items
    });
  }
});
var SchemaField = React.createClass({
  displayName: "SchemaField",
  getDefaultProps: function getDefaultProps() {
    return {
      model: 'ZNPluginWorkflowSchemaField'
    };
  },
  getInitialState: function getInitialState() {
    return {
      data: zn.store.post('/zn.plugin.admin/model/paging', {
        model: this.props.model,
        where: this.props.schema_uuid ? {
          schema_uuid: this.props.schema_uuid
        } : null
      }),
      items: [{
        title: '表单名称(title)',
        name: 'title',
        width: 120
      }, {
        title: '表单名(name)',
        name: 'name',
        width: 100
      }, {
        title: '表单类型',
        name: 'type',
        width: 100
      }, {
        title: '资源ID(var_id)',
        name: 'var_id',
        width: 100
      }, {
        title: '宽度(width)',
        name: 'width',
        width: 100
      }, {
        title: '数据源(data)',
        name: 'data',
        width: 160
      }, {
        title: '属性(props)',
        name: 'props',
        width: 200
      }, {
        title: '字段类型',
        name: 'field_type',
        width: 140
      }, {
        title: '字段函数',
        name: 'field_convert',
        width: 140
      }, {
        title: '字段默认值',
        name: 'field_default',
        width: 100
      }, {
        title: '说明',
        name: 'zn_note'
      }],
      formItems: [{
        title: '字段类型',
        type: 'Input',
        name: 'field_type'
      }, {
        title: '字段函数',
        type: 'Input',
        name: 'field_convert'
      }, {
        title: '字段默认值',
        type: 'Input',
        name: 'field_default'
      }, //{ title: '字段描述', type: 'Textarea', name: 'field_detail' },
      {
        title: '表单名称(title)',
        type: 'Input',
        name: 'title'
      }, {
        title: '表单名(name)',
        type: 'Input',
        name: 'name'
      }, {
        title: '表单类型',
        type: 'Menu',
        name: 'type',
        data: ['Input', 'Select', 'Label', 'ImageUploader', 'FileUploader', 'Checkbox', 'CheckboxGroup', 'Radio', 'RadioGroup', 'Textarea', 'Richarea']
      }, {
        title: '资源ID(var_id)',
        type: 'Input',
        attrs: {
          type: 'number'
        },
        name: 'var_id'
      }, {
        title: '宽度(width)',
        type: 'Input',
        attrs: {
          type: 'number'
        },
        name: 'width'
      }, {
        title: '数据源(data)',
        type: 'Textarea',
        name: 'data'
      }, {
        title: '属性(props)',
        type: 'Textarea',
        name: 'props'
      }, {
        title: '说明',
        type: 'Textarea',
        name: 'zn_note'
      }],
      toolbarItems: [{
        text: '添加',
        method: '__addItem',
        icon: 'fa-plus',
        style: {
          marginRight: 5
        }
      }, {
        text: '删除',
        method: '__removeItems',
        status: 'danger',
        icon: 'fa-remove',
        style: {
          marginRight: 5
        }
      }]
    };
  },
  componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
    if (nextProps.schema_uuid) {
      this.state.data.extend({
        where: {
          schema_uuid: nextProps.schema_uuid
        }
      }).refresh();
    }
  },
  __doSuccess: function __doSuccess() {
    this.state.data.refresh();
    zn.notification.success('操作成功');
    zn.modal.close();
    return false;
  },
  __addItem: function __addItem(event) {
    event.stopPropagation();
    /*
    zn.popover.render(<zn.react.Form
    	action='/zn.plugin.admin/model/insert'
    	merge='values'
    	hiddens={{ schema_uuid: this.props.schema_uuid }}
    	exts={{ model: this.props.model }}
    	onSubmitSuccess={this.__doSuccess}
    	items={this.state.formItems} />, zn.extend({
    	event: event,
    	reset: true,
    	target: event.target,
    	style: {overflow: 'auto'}
    }, this.props));*/

    zn.dialog({
      title: '新增',
      content: React.createElement(zn.react.Form, {
        action: "/zn.plugin.admin/model/insert",
        merge: "values",
        hiddens: {
          schema_uuid: this.props.schema_uuid
        },
        exts: {
          model: this.props.model
        },
        onSubmitSuccess: this.__doSuccess,
        items: this.state.formItems
      })
    });
  },
  __updateItem: function __updateItem(data) {
    zn.dialog({
      title: '更新',
      content: React.createElement(zn.react.Form, {
        merge: "updates",
        action: "/zn.plugin.admin/model/update",
        exts: {
          model: this.props.model,
          where: {
            id: data.id
          }
        },
        value: data,
        onSubmitSuccess: this.__doSuccess,
        items: this.state.formItems
      })
    });
  },
  __removeItems: function __removeItems() {
    var _self = this,
        _values = this.refs.table.getValue();

    if (_values && _values.length) {
      zn.confirm('确认删除这' + _values.length + '个数据吗？', '提示', function () {
        zn.http.post('/zn.plugin.admin/model/delete', {
          model: this.props.model,
          where: "id in (" + _values.join(',') + ")"
        }).then(function (data) {
          if (data.status == 200) {
            zn.notification.success('删除成功');

            _self.state.data.refresh();
          } else {
            zn.notification.error('调用失败: ' + data.result);
          }
        }, function () {
          zn.notification.error('服务器错误: 删除失败');
        });
      }.bind(this));
    } else {
      zn.notification.warning('请先选择要删除的数据');
    }
  },
  __onToolbarClick: function __onToolbarClick(item, icon, icons, event) {
    this[item.method] && this[item.method](event);
  },
  __onTableColumnRender: function __onTableColumnRender(rowIndex, columnIndex, data, item, value) {
    var _this2 = this;

    switch (columnIndex) {
      case 1:
        return React.createElement("span", null, React.createElement("i", {
          className: "fa fa-edit zr-padding-3",
          onClick: function onClick() {
            return _this2.__updateItem(data);
          }
        }), value);
    }

    return value;
  },
  render: function render() {
    return React.createElement(zn.react.PagerView, {
      ref: "table",
      view: "Table",
      checkbox: 50,
      showHeader: true,
      toolbarItems: this.state.toolbarItems,
      onToolbarClick: this.__onToolbarClick,
      columnRender: this.__onTableColumnRender,
      data: this.state.data,
      dataFixed: true,
      items: this.state.items
    });
  }
});
var SchemaInfo = React.createClass({
  displayName: "SchemaInfo",
  getInitialState: function getInitialState() {
    return {
      currIndex: 0
    };
  },
  __doSuccess: function __doSuccess() {
    zn.notification.success('修改成功');
    return false;
  },
  __renderBody: function __renderBody() {
    switch (this.state.currIndex) {
      case 0:
        return React.createElement(zn.react.Form, {
          merge: "updates",
          action: "/zn.plugin.admin/model/update",
          exts: {
            model: 'ZNPluginWorkflowSchema',
            where: {
              uuid: this.props.id
            }
          },
          value: zn.store.post('/zn.plugin.workflow/wfschema/getNodeSchema', {
            id: this.props.id
          }),
          onSubmitSuccess: this.__doSuccess,
          items: [{
            title: 'UUID',
            type: 'Label',
            name: 'uuid'
          }, {
            title: '名称',
            type: 'Label',
            name: 'zn_title'
          }, {
            title: '类型',
            type: 'Select',
            disabled: true,
            name: 'type',
            data: TYPE_ARRAY
          }, {
            title: '权限逻辑',
            type: 'Select',
            name: 'rights_logic',
            data: RIGHTS_LOGICS
          }, {
            title: '权限用户',
            type: zn.plugin.admin.UserSelector,
            name: 'zn_rights_observe_users'
          }, {
            title: '权限角色',
            type: zn.plugin.admin.RoleSelector,
            name: 'zn_rights_observe_roles'
          }, {
            title: '属性',
            type: 'Textarea',
            name: 'attrs'
          }]
        });

      case 1:
        return React.createElement(SchemaAction, {
          schema_uuid: this.props.id
        });

      case 2:
        return React.createElement(SchemaField, {
          schema_uuid: this.props.id
        });
    }
  },
  render: function render() {
    var _this3 = this;

    return React.createElement("div", {
      className: "zr-flex-layout"
    }, React.createElement(zn.react.ListView, {
      className: "layout-header zr-tab-ios",
      style: {
        margin: 5
      },
      selectMode: "radio",
      textKey: "text",
      valueKey: "index",
      onClick: function onClick(value) {
        return _this3.setState({
          currIndex: value.item.index
        });
      },
      value: this.state.currIndex,
      data: [{
        index: 0,
        text: '字段信息',
        icon: 'fa-list-alt'
      }, {
        index: 1,
        text: '节点规则',
        icon: 'fa-table'
      }, {
        index: 2,
        text: '节点表单',
        icon: 'fa-list'
      }]
    }), React.createElement("div", {
      className: "layout-body"
    }, this.__renderBody()));
  }
});
module.exports = React.createClass({
  displayName: "exports",
  getInitialState: function getInitialState() {
    return {
      info: {},
      json_schema: {},
      schemas: null,
      toolBarItems: []
    };
  },
  componentDidMount: function componentDidMount() {
    this.__loadInfo();
  },
  componentDidUpdate: function componentDidUpdate(prevProps, prevState) {
    if (prevProps.indexId != this.props.indexId) {
      this.__loadInfo();
    }
  },
  __onToolbarClick: function __onToolbarClick(obj, nodeProps) {
    switch (obj.name) {
      case 'workflow.schema.init':
        zn.dialog({
          title: '初始化流程画布',
          content: React.createElement(zn.react.Form, {
            onSubmitBefore: function (value) {
              this.setState({
                json_schema: {
                  nodes: [{
                    x: 100,
                    y: 100,
                    data: {
                      title: value.value || '开始',
                      type: 'begin'
                    }
                  }],
                  links: []
                }
              });
              return zn.modal.close(), false;
            }.bind(this),
            items: [{
              title: '节点名称',
              name: 'value',
              type: 'Input'
            }, {
              title: '节点类型',
              name: 'type',
              type: 'Label',
              value: '开始节点'
            }]
          })
        });
        break;

      case 'workflow.schema.save':
        this.__save();

        break;

      case 'workflow.schema.define':
        this.__define();

        break;

      case 'workflow.schema.node.define.edit':
        this.__editDefine(nodeProps);

        break;

      case 'workflow.schema.node.delete':
        this.__onRemoveNode(nodeProps);

        break;
    }
  },
  __editDefine: function __editDefine(nodeProps) {
    zn.dialog({
      title: '编辑流程节点',
      content: React.createElement(SchemaInfo, nodeProps)
    });
  },
  __onRemoveNode: function __onRemoveNode(nodeProps) {
    zn.confirm('确定删除改节点吗？', '提示', function () {
      if (nodeProps.data.type == 'begin') {
        return zn.toast.error('注意：开始节点不能被删除！'), false;
      }

      this.refs.flowcanvas.deleteNodeById(nodeProps.id);
    }.bind(this));
  },
  __save: function __save() {
    zn.preloader.open({
      title: '保存中...'
    });
    zn.http.post('/zn.plugin.admin/model/update', {
      model: 'ZNPluginWorkflowIndex',
      updates: {
        json_schema: JSON.stringify(this.refs.flowcanvas.getData())
      },
      where: {
        id: this.props.indexId
      }
    }).then(function (data) {
      if (data.status == 200) {
        zn.notification.success('保存成功');
      } else {
        zn.notification.success('保存失败：' + data.result);
      }

      zn.preloader.close();
    }, function (data) {
      zn.notification.success('保存失败：服务器错误');
      zn.preloader.close();
    });
  },
  __define: function __define() {
    zn.confirm('确定生成流程定义吗？', '提示', function () {
      zn.preloader.open({
        title: '生成中...'
      });
      zn.http.post('/zn.plugin.workflow/wfindex/toDefinition', {
        indexId: this.props.indexId
      }).then(function (data) {
        if (data.status == 200) {
          zn.notification.success('生成流程定义成功');

          this.__loadInfo();
        } else {
          zn.notification.error('生成失败：' + data.result);
        }

        zn.preloader.close();
      }.bind(this), function () {
        zn.preloader.close();
      });
    }.bind(this));
  },
  __loadInfo: function __loadInfo() {
    zn.preloader.open({
      title: '加载数据...'
    });
    zn.http.post('/zn.plugin.workflow/wfindex/getIndexSchema', {
      indexId: this.props.indexId
    }).then(function (data) {
      if (data.status == 200) {
        var _states = {},
            _data = data.result,
            _tbItems = [{
          tooltip: '初始化流程画布',
          name: 'workflow.schema.init',
          icon: 'fa-paint-brush'
        }, {
          tooltip: '保存画布',
          name: 'workflow.schema.save',
          icon: 'fa-floppy-o'
        }, {
          tooltip: '生成流程定义',
          name: 'workflow.schema.define',
          icon: 'fa-code-fork'
        }];
        _states = {
          info: _data.info,
          json_schema: _data.info.json_schema,
          schemas: _data.schemas,
          toolBarItems: _tbItems
        };

        if (this.state.currData) {
          var _actions = [{
            tooltip: '删除节点',
            name: 'workflow.schema.node.delete',
            icon: 'fa-remove'
          }];

          if (_states.schemas && _states.schemas.length) {
            _actions.push({
              text: '编辑节点',
              name: 'workflow.schema.node.define.edit',
              icon: 'fa-edit'
            });
          }

          _states.subToolbarItems = _actions;
        }

        this.setState(_states);
      } else {
        zn.toast.error(data.result);
      }

      zn.preloader.close();
    }.bind(this), function () {
      zn.toast.error('加载失败');
      zn.preloader.close();
    });
  },
  __onNodeTypeClick: function __onNodeTypeClick(event, node, props) {
    //event.stopPropagation();
    console.log('xxx');
  },
  __nodeRender: function __nodeRender(data, node) {
    var _this4 = this;

    return React.createElement("div", {
      className: "wf-node"
    }, React.createElement("div", {
      className: "node-type type-" + data.type,
      onClick: function onClick(event) {
        return _this4.__onNodeTypeClick(event, node, data);
      }
    }, NODE_TYPES[data.type], React.createElement("i", {
      className: "action fa fa-ellipsis-v"
    })), React.createElement("div", {
      className: "node-title"
    }, data.title));
  },
  __onNodeDragEnd: function __onNodeDragEnd() {//this.__save();
  },
  __onNodeEditDragEnd: function __onNodeEditDragEnd(node, data) {
    var _parentPosition = node._parentPosition,
        _vector = {
      x: data.currX - _parentPosition.x,
      y: data.currY - _parentPosition.y
    };
    zn.dialog({
      title: '编辑',
      content: React.createElement(zn.react.Form, {
        onSubmitBefore: function onSubmitBefore(data, form) {
          _vector.data = data;
          node.props.canvas.addNode(_vector, node);
          return zn.modal.close(), false;
        },
        items: [{
          title: '节点名称',
          type: 'Input',
          name: 'title',
          required: true
        }, {
          title: '节点类型',
          type: 'Select',
          name: 'type',
          data: TYPE_ARRAY,
          required: true
        }]
      })
    });
  },
  getParent: function getParent(target) {
    if (target.classList.contains('wf-node')) {
      return target;
    } else {
      return this.getParent(target.parentNode);
    }
  },
  __onNodeClick: function __onNodeClick(event, node, props) {
    if (this.props.disabled) {
      return;
    }

    var _actions = [{
      tooltip: '删除节点',
      name: 'workflow.schema.node.delete',
      icon: 'fa-remove'
    }];

    if (this.state.schemas && this.state.schemas.length) {//_actions.push({ text: '编辑节点', name: 'workflow.schema.node.define.edit', icon: 'fa-edit' });
    }

    this.setState({
      currData: props,
      subToolbarItems: _actions
    });
  },
  render: function render() {
    var _this7 = this;

    return React.createElement("div", {
      className: "zn-plugin-workflow-schema-designer"
    }, React.createElement("div", {
      className: "tool-bars",
      onClick: function onClick(event) {
        return event.stopPropagation();
      }
    }, this.state.toolBarItems.map(function (item) {
      var _this5 = this;

      return React.createElement("i", {
        onClick: function onClick() {
          return _this5.__onToolbarClick(item);
        },
        "data-tooltip": item.tooltip,
        className: "fa " + item.icon
      });
    }.bind(this)), React.createElement("i", {
      className: "separator"
    }), this.state.currData && this.state.subToolbarItems.map(function (item) {
      var _this6 = this;

      return React.createElement("i", {
        onClick: function onClick() {
          return _this6.__onToolbarClick(item, _this6.state.currData);
        },
        "data-tooltip": item.tooltip,
        className: "fa " + item.icon
      });
    }.bind(this))), React.createElement(zn.react.FlowCanvas, {
      ref: "flowcanvas",
      onNodeEditDragEnd: this.__onNodeEditDragEnd,
      onNodeDragEnd: this.__onNodeDragEnd,
      onNodeClick: this.__onNodeClick,
      nodeRender: this.__nodeRender,
      data: this.state.json_schema
    }), this.state.currData && React.createElement("div", {
      className: "schema-info"
    }, React.createElement(SchemaInfo, this.state.currData), React.createElement("i", {
      title: "\u5173\u95ED",
      className: "close fa fa-remove",
      onClick: function onClick() {
        return _this7.setState({
          currData: null
        });
      }
    })));
  }
});