require('./WFInstance.less');
var React = require('react');
var BaseInfo = require('./instance/BaseInfo');
var FlowStep = require('./instance/FlowStep');
var FlowTableInfo = require('./instance/FlowTableInfo');
module.exports = React.createClass({
	getInitialState: function () {
		return {
			base: null,
			current: null,
			data: null,
			table_fields: [],
			table_data: []
		};
	},
	componentDidMount: function (){
		this.__loadInfo();
	},
	componentDidUpdate: function (prevProps, prevState){
		if(prevProps.instanceId!=this.props.instanceId){
			this.__loadInfo();
		}
	},
	__loadInfo: function (){
		if(this.props.instanceId){
			zn.preloader.open({ title: '加载数据...' });
			zn.http.post('/zn.plugin.workflow/wfinstance/getInstance', {
				instanceId: this.props.instanceId
			}).then(function (data){
				if(data.status==200){
					this.setState(data.result);
				}else {
					zn.notification.error(data.result);
					if(data.status==401){
						zn.react.session.doHome();
					}
				}
				zn.preloader.close();
			}.bind(this), function (){
				zn.notification.error('加载失败');
				zn.preloader.close();
			});
		}
	},
	__cancel: function (){
		zn.confirm('确定取消流程吗？', '提示', function (){
			zn.preloader.open({ title: '流程取消中...' });
			zn.http.post('/zn.plugin.workflow/wfinstance/cancel', {
				instanceId: this.state.base.id
			}).then(function (data){
				if(data.status==200){
					this.__loadInfo();
				}else {
					zn.notification.error(data.result);
				}
				zn.preloader.close();
			}.bind(this), function (){
				zn.notification.error('取消失败');
				zn.preloader.close();
			});
		}.bind(this));
	},
	__remove: function (){
		zn.confirm('删除流程将删除流程相关的所有数据, 确定删除吗？', '提示', function (){
			zn.preloader.open({ title: '流程删除中...' });
			zn.http.post('/zn.plugin.workflow/wfinstance/remove', {
				instanceId: this.state.base.id
			}).then(function (data){
				if(data.status==200){
					zn.notification.success('删除成功');
					zn.react.session.back();
				}else {
					zn.notification.error(data.result);
				}
				zn.preloader.close();
			}.bind(this), function (){
				zn.notification.error('删除失败');
				zn.preloader.close();
			});
		}.bind(this));
	},
	__start: function (){
		zn.confirm('确定打开流程吗？', '提示', function (){
			zn.preloader.open({ title: '流程打开中...' });
			zn.http.post('/zn.plugin.workflow/wfinstance/start', {
				instanceId: this.state.base.id
			}).then(function (data){
				if(data.status==200){
					this.__loadInfo();
				}else {
					zn.notification.error(data.result);
				}
				zn.preloader.close();
			}.bind(this), function (){
				zn.notification.error('打开失败');
				zn.preloader.close();
			});
		}.bind(this));
	},
	__renderAction: function (){
		if(this.state.base.isCreator){
			return <div style={{display: 'inline-block'}}>
				{
					this.state.base.closed ? <a style={{ color: '#1289d8', marginLeft: 10, fontSize: 14 }} onClick={this.__start} href="javascript:void(0);">打开</a> :
					<a style={{ color: '#f00', marginLeft: 10, fontSize: 14 }} onClick={this.__cancel} href="javascript:void(0);">关闭</a>
				}
				{
					this.state.base.state != -1 && <a style={{ color: '#f00', marginLeft: 10, fontSize: 14 }} onClick={this.__remove} href="javascript:void(0);">删除</a>
				}
			</div>;
		}else {
			return null;
		}
	},
	render:function(){
		if(!this.state.current || !this.state.base){
			return <div className="zn-plugin-workflow-instance" >
				<div className="not-exist">流程实例Id({this.props.instanceId})不存在, 请核实后再试!</div>
			</div>;
		}
		if(this.state.base.zn_rights_enabled && !this.state.base.doRights && !this.state.base.doObserveRights){
			return <div className="zn-plugin-workflow-instance" >
				<div className="not-exist">此流程您暂无【浏览】及【操作】权限, 请联系系统管理员后再重试!</div>
			</div>;
		}
		return (
			<div className={"zn-plugin-workflow-instance closed-" + this.state.base.closed} >
				<div className="index-title">
					<span>{this.state.base.index_id_convert}</span>
					<span> {this.state.base.code}</span>
					{this.__renderAction()}
				</div>
				<div style={{ display: 'flex', justifyContent: 'space-between', padding: '0px 3px' }}>
					<div><span>申请人：</span><span style={{ fontWeight: 200 }}>{this.state.base.zn_create_user_convert}</span></div>
					<div><span>申请时间：</span><span style={{ fontWeight: 200 }}>{this.state.base.zn_create_time}</span></div>
				</div>
				{(this.state.table_fields && !!this.state.table_fields.length) && <FlowTableInfo fields={this.state.table_fields} data={this.state.table_data} />}
				{(this.state.base && !this.state.base.closed && !!this.state.base.doRights && this.state.current) && <BaseInfo value={this.props.value} onNextSuccess={this.__loadInfo} base={this.state.base} current={this.state.current} />}
				{(this.state.data && this.state.current) && <FlowStep instanceId={this.props.instanceId} data={this.state.data} current={this.state.current} onRemarkSuccess={this.__loadInfo} />}
				{
					this.state.base.zn_note && <div className="zn-plugin-workflow-panel" >
						<div className="title">流程备注</div>
						<div className="content">
							<div style={{ minHeight: 60 }}>{this.state.base.zn_note}</div>
						</div>
					</div>
				}
			</div>
		);
	}
});
